// Lawson Labs, Inc.
// 3217 Phoenixville Pike
// Malvern, PA  19355
// 800 321-5355 or 610 725-8800             
// Last updated (made 32-bit) 6-3-99
// Programmer (32-bit) - Tim Van Dusen 
// filename: NT_m202a




//=============================================================================
// Structured data shared with DLL
//=============================================================================

struct T_M202{
	short		comPort;	  		//com port number 1-4
	double	scanInterval;	//scan interval in seconds
	short		numChannels;	//number of active channels for scanning
	short		tScan;			//type of scan 0 = single channel
									//					1 = normal
									//					2 = normal with calibration
	short		chan[6];			//control code array for scanning
	short		singleChan;    //single channel scanning channel
	short    lastDigInp; 	//last digital input value
	short 	version;			//Model 202 version
	short 	channel;			//currently selected channel (used in GetConversion)
	short 	subChan;			//0 = dacs disabled, 8 = dacs enabled
	long	   rawCount;		//current raw count returned from Model 202
	double	voltage;			//current voltage
	int 	   digOutWord;		//value sent to Model 202 for digital output (0-255)
	short 	wordCnt;			//number of words in conversion (2 or 3)
	short 	biPolar;			//0 = unipolar  1 = bipolar
	int 	   errorType;		//error code returned from DLL
	short 	filter;			//model 201 specific element
	short 	gain;				//gain factor (powers of 2)
	float 	rate;				//sample rate (10 - 1027 Hz)
	short 	modeRegLo;		//mode registers for signon
	short 	modeRegMid;
	short 	modeRegHi;
	short		baud;				//selected baud rate code (0 - 5)
	short		sleeping;		//model 201 specific element
	short		standBy;       //0 = Model 202 out of standby  1 = Model 202 in standby
	int		timeBaseCnt;	//must be a positive value
	unsigned int    average;  //remote averaging (powers of 2)
	//transmit/receive queue sizes as used by Windows COMM driver
	unsigned int    TXQUEUESIZE;
	unsigned int    RXQUEUESIZE;
} M202, *pM202;





//=============================================================================
//Global Variables
//=============================================================================

BYTE			NewScanBuff[128];
BYTE			*pNewScanBuff;		
HINSTANCE 	hLibrary;
HANDLE		idComDev;   //  device channel to com port as used by Windows
HANDLE		hOutCon, hInCon, hLibM201;
DWORD			lpNumberOfCharsWritten;
COORD			pos, dwCursorPosition;
CONSOLE_CURSOR_INFO	ConsoleCursorInfo;
CONSOLE_SCREEN_BUFFER_INFO lpConsoleScreenBufferInfo;





//=============================================================================
//macros
//=============================================================================

// following macro allows use of gotoxy and clrscr 
// functions used by previous 16-bit and Win32s code
#define gotoxy(x, y) pos.X = x; pos.Y = y - 1;	\
SetConsoleCursorPosition(hOutCon, pos);
#define clrscr() pos.X = 0; pos.Y = 0;		\
FillConsoleOutputCharacter(hOutCon, ' ',	\
80*25, pos, &lpNumberOfCharsWritten);





//=============================================================================
//prototypes for local functions
//=============================================================================

// dynamically load/unload DLL
BOOLEAN LoadLibraries(void);
BOOLEAN UnloadLibraries(void);

// check DLL for functions that will be used
BOOLEAN CheckLibFuncs(void);


// turns on cursor in Win32 console mode
void    SetWin32CursorOn(void);

// turns off cursor in Win32 console mode
void    SetWin32CursorOff(void);

void    AnalogOut(double *dacAvolts, double *dacBvolts);

// read 202EWIN.CFG file from disk in current directory
short   ReadConfig(short *localAvg);

// write 202EWIN.CFG file to disk in current directory
short   WriteConfig(short *localAvg);

// select the scan interval
short   CalcScanTime(void);

// displays the scanning menu (returns 1 if set to start scanning)
short   ScanMenu(void);

// select the sampling rate for the A/D
void    SelectRate(void);

//   Get starting & ending control codes for a scanning channel
short   GetChannel(int value);

// display polled mode functions menu
void    PrintCommandMenu(int Ch, short localAvg, short checkSum);

// perform selected function
void    DoCommand(int Ch, double *max, double *min, short *localAvg,
						short *checkSum, double *dacAvolts, double *dacBvolts);

// calculate standard deviation
void    StandardDev(double *stDv, double *rmsBits, double *sdTot,
						  double *runningAvg, double *sd, int *sdCount);





//=============================================================================
//prototypes for external DLL functions
//=============================================================================

typedef short(*DLL_SET_PTRS)(T_M202 *M202, BYTE *pNewScanBuff);
DLL_SET_PTRS SetPtrs;

//This function will sign on to the Model 202 using the settings in M202
//If could not sign on, closes port and returns 0. Otherwise, returns 1,
//port is open and the Model 202 is active.

typedef short(*DLL_SIGN_ON)();
DLL_SIGN_ON SignOn;

//take a raw count and scale in volts
typedef double(*DLL_COUNTS_2_VOLTS)();
DLL_COUNTS_2_VOLTS Counts2Volts;

//Get a digital input from the Model 202.  Returns 0 if Model 202 did not
//echo back the Get Digital Input token.
typedef short(*DLL_GET_DIG_IN)(void);
DLL_GET_DIG_IN GetDigIn;

//Get the Model 202 version.  Returns 0 if Model 202 did not echo back the
//version token.
typedef short(*DLL_GET_VER)(void);
DLL_GET_VER GetVer;

//Send a digital output to the Model 202
typedef short(*DLL_SET_DIG_OUT)(int digOutWord);
DLL_SET_DIG_OUT SetDigOut; 

//Set averaging in the Model 202 (powers of 2)
typedef short(*DLL_REMOTE_AVG)(void);
DLL_REMOTE_AVG RemoteAvg;

//Perform a full scale calibration on the currently selected channel
typedef short(*DLL_FULL_SCALE_CAL)(void);
DLL_FULL_SCALE_CAL FullScaleCal; 

//Perform an offset calibration on the currently selected channel
typedef short(*DLL_OFFSET_CAL)(void);
DLL_OFFSET_CAL OffsetCal;

//Perform a system calibration (full scale on channel 6, offset on channel 7)
typedef short(*DLL_SYS_CAL)(void);
DLL_SYS_CAL SysCal;

//Get a conversion from the Model 202 on the currently selected channel
typedef short(*DLL_GET_CONVERSION)(void);
DLL_GET_CONVERSION GetConversion;

//Select channel and subchannel from M2021.channel and M202.subChan
typedef short(*DLL_SET_CHAN_CODE)(void);
DLL_SET_CHAN_CODE SetChanCode;

//Start scanning using selected scan type. Returns 0 if scan token is not
//echoed back.
typedef short(*DLL_START_SCANNING)(void);
DLL_START_SCANNING StartScanning;

//Get one scan from currently selected comm port buffer. It will wait the
//calculated time for the scan to come in. Returns 0 if failed. Returns type
//of failure in M202.errorType.
//M202.errorType = 1		Start scan framing failure
//M202.errorType = 2		not enough chars, possible communications error
//M202.errorType = 3		Comm port buffer overflow
//M202.errorType = 0xA  Scan rate too fast for this scan type
typedef short(*DLL_GET_ONE_SCAN)();
DLL_GET_ONE_SCAN GetOneScan;

//Tell the Model 201 to stop scanning. Returns 0 if Stop scan token is not
//echoed back
typedef short(*DLL_STOP_SCANNING)(void);
DLL_STOP_SCANNING StopScanning;

//Get counts from ScanBuffer, scale them in volts, and remove counts from
//ScanBuffer
typedef void(*DLL_GCONV)(void);
DLL_GCONV Gconv;

//Remove size elements from ScanBuffer
typedef void(*DLL_REMOVE_FROM_BUFFER)(short size);
DLL_REMOVE_FROM_BUFFER RemFromBuffer;

//Set the filter  0 = 4 Hz   1 = 40 Hz   2 = 400 Hz
typedef void(*DLL_SET_FILTER)(short filter);
DLL_SET_FILTER SetFilter;

//Send mode registers to Model 201. Returns 0 if the registers read back
//incorrectly
typedef short(*DLL_SEND_AD_MODE)(void);
DLL_SEND_AD_MODE SendADMode;


//Get and compare Model 201 checksum. Returns 0 if failed, 1 if checksum is OK
typedef short(*DLL_GET_CHECK_SUM)(void);
DLL_GET_CHECK_SUM GetCheckSum;

//Send the stop scan command to stop a scan if scanning, send master reset,
//and close the currently selected comm port.
typedef short(*DLL_CLOSE_DEVICE)(void);
DLL_CLOSE_DEVICE CloseDevice; 

//Put the Model 202 in sleep mode
typedef short(*DLL_GOTO_SLEEP)(void);
DLL_GOTO_SLEEP GotoSleep;


typedef short(*DLL_WRITE_CNTR)(int cntPort, int cntrCount);
DLL_WRITE_CNTR WriteCntr;


//Wait the filter settling time
typedef void(*DLL_FILTER_DELAY)(short filter);
DLL_FILTER_DELAY FilterDelay;

//Check the selected scan interval to see if it is in limits
typedef short(*DLL_CHECK_SCAN_TIME)(double *maxTime, double *minTime);
DLL_CHECK_SCAN_TIME CheckScanTime; 

//The next two functions only need to be called if handling more than one
//Model 202 at a time.

//Get the comm handle after Signon
typedef void(*DLL_GET_COM_HANDLE)(HANDLE *IdComDev);
DLL_GET_COM_HANDLE GetComHandle;
